<?php
    interface PasswordInterface {
        const PASSWORD_MAX_LENGTH = 512;
        public function hash($password);
        public function check($password, $hash);
    }

    class CryptXY {
        public static function randomBytes($count) {
            if(function_exists('openssl_random_pseudo_bytes')) {
                return openssl_random_pseudo_bytes($count);
            } elseif(function_exists('mcrypt_create_iv')) {
                return mcrypt_create_iv($count, MCRYPT_DEV_URANDOM);
            } elseif(function_exists('random_bytes')) {
                return random_bytes($count);
            } else {
                throw new Exception('Random number generator not available.');
            }
        }

        public static function hmacBase64($data, $key) {
            if(!is_scalar($data) || !is_scalar($key)) {
                throw new InvalidArgumentException('Both parameters passed to CryptXY::hmacBase64 must be scalar values.');
            }
            $hmac = base64_encode(hash_hmac('sha256', $data, $key, TRUE));
            return str_replace(['+', '/', '='], ['-', '_', ''], $hmac);
        }

        public static function hashBase64($data) {
            $hash = base64_encode(hash('sha256', $data, TRUE));
            return str_replace(['+', '/', '='], ['-', '_', ''], $hash);
        }

        public static function hashEquals($known_string, $user_string) {
            if(function_exists('hash_equals')) {
                return hash_equals($known_string, $user_string);
            } else {
                if(!is_string($known_string)) {
                    trigger_error(sprintf("Expected known_string to be a string, %s given", gettype($known_string)), E_USER_WARNING);
                    return FALSE;
                }
                if(!is_string($user_string)) {
                    trigger_error(sprintf("Expected user_string to be a string, %s given", gettype($user_string)), E_USER_WARNING);
                    return FALSE;
                }
                $known_len = strlen($known_string);
                if($known_len !== strlen($user_string)) {
                    return FALSE;
                }
                $result = 0;
                for ($i = 0; $i < $known_len; $i++) {
                    $result |= (ord($known_string[$i]) ^ ord($user_string[$i]));
                }
                return $result === 0;
            }
        }

        public static function randomBytesBase64($count = 32) {
            return str_replace(['+', '/', '='], ['-', '_', ''], base64_encode(static::randomBytes($count)));
        }
    }
}
if(!class_exists("PhpassHashedPassword")) {
    class PhpassHashedPassword implements PasswordInterface {
        const MIN_HASH_COUNT = 7;
        const MAX_HASH_COUNT = 30;
        const HASH_LENGTH = 55;
        public static $ITOA64 = './0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz';
        protected $countLog2;

        function __construct($countLog2) {
            $this->countLog2 = $this->enforceLog2Boundaries($countLog2);
        }

        protected function base64Encode($input, $count) {
            $output = '';
            $i = 0;
            do {
                $value = ord($input[$i++]);
                $output .= static::$ITOA64[$value & 0x3f];
                if($i < $count) {
                    $value |= ord($input[$i]) << 8;
                }
                $output .= static::$ITOA64[($value >> 6) & 0x3f];
                if($i++ >= $count) {
                    break;
                }
                if($i < $count) {
                    $value |= ord($input[$i]) << 16;
                }
                $output .= static::$ITOA64[($value >> 12) & 0x3f];
                if($i++ >= $count) {
                    break;
                }
                $output .= static::$ITOA64[($value >> 18) & 0x3f];
            } while ($i < $count);

            return $output;
        }

        protected function generateSalt() {
            $output = '$S$';
            $output .= static::$ITOA64[$this->countLog2];
            $output .= $this->base64Encode(CryptXY::randomBytes(6), 6);
            return $output;
        }

        protected function enforceLog2Boundaries($count_log2) {
            if($count_log2 < static::MIN_HASH_COUNT) {
                return static::MIN_HASH_COUNT;
            } elseif($count_log2 > static::MAX_HASH_COUNT) {
                return static::MAX_HASH_COUNT;
            }
            return (int) $count_log2;
        }

        protected function cryptxy($algo, $password, $setting) {
            if(strlen($password) > PasswordInterface::PASSWORD_MAX_LENGTH) {
                return FALSE;
            }
            $setting = substr($setting, 0, 12);
            if($setting[0] != '$' || $setting[2] != '$') {
                return FALSE;
            }
            $count_log2 = $this->getCountLog2($setting);
            if($count_log2 != $this->enforceLog2Boundaries($count_log2)) {
                return FALSE;
            }
            $salt = substr($setting, 4, 8);
            if(strlen($salt) != 8) {
                return FALSE;
            }
            $count = 1 << $count_log2;
            $hash = hash($algo, $salt . $password, TRUE);
            do {
                $hash = hash($algo, $hash . $password, TRUE);
            } while (--$count);
            $len = strlen($hash);
            $output = $setting . $this->base64Encode($hash, $len);
            $expected = 12 + ceil((8 * $len) / 6);
            return (strlen($output) == $expected) ? substr($output, 0, static::HASH_LENGTH) : FALSE;
        }

        public function getCountLog2($setting) {
            return strpos(static::$ITOA64, $setting[3]);
        }
        public function hash($password) {
            return $this->cryptxy('sha512', $password, $this->generateSalt());
        }
        public function check($password, $hash) {
            if(substr($hash, 0, 2) == 'U$') {
                $stored_hash = substr($hash, 1);
                $password = md5($password);
            } else {
                $stored_hash = $hash;
            }
            $type = substr($stored_hash, 0, 3);
            switch ($type) {
                case '$S$':
                    // A normal Drupal 7 password using sha512.
                    $computed_hash = $this->cryptxy('sha512', $password, $stored_hash);
                    break;
                case '$H$':
                    // phpBB3 uses "$H$" for the same thing as "$P$".
                case '$P$':
                    // A phpass password generated using md5.  This is an
                    // imported password or from an earlier Drupal version.
                    $computed_hash = $this->cryptxy('md5', $password, $stored_hash);
                    break;
                default:
                    return FALSE;
            }
            return $computed_hash && CryptXY::hashEquals($stored_hash, $computed_hash);
        }
    }
?>